package at.xaxa.gamelibraryapp.ui

import androidx.compose.foundation.background
import androidx.compose.foundation.border
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.requiredHeight
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.List
import androidx.compose.material.icons.filled.Add
import androidx.compose.material3.Icon
import androidx.compose.material3.NavigationBarItem
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.em
import androidx.compose.ui.unit.sp
import androidx.navigation.NavType
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.currentBackStackEntryAsState
import androidx.navigation.compose.rememberNavController
import androidx.navigation.navArgument
import at.xaxa.gamelibraryapp.ui.Details.DetailView
import at.xaxa.gamelibraryapp.ui.GameList.GameList
import at.xaxa.gamelibraryapp.ui.Search.SearchList
import coil3.compose.AsyncImage

enum class LibraryRoutes(val route: String) {
    List("list"),
    Add("add"),
    Detail("game/{gameId}")
}

@Composable
fun LibraryApp(modifier: Modifier = Modifier) {
    val navController = rememberNavController()
    val currentRoute = navController.currentBackStackEntryAsState().value?.destination?.route

    Scaffold(
        bottomBar = {
            // Hide bottom bar only when in the Detail view
            if (currentRoute != LibraryRoutes.Detail.route) {
                androidx.compose.material3.NavigationBar {
                    NavigationBarItem(
                        icon = { Icon(Icons.AutoMirrored.Filled.List, contentDescription = "List") },
                        label = { Text("List") },
                        selected = currentRoute == LibraryRoutes.List.route,
                        onClick = { navController.navigate(LibraryRoutes.List.route) }
                    )
                    NavigationBarItem(
                        icon = { Icon(Icons.Default.Add, contentDescription = "Add") },
                        label = { Text("Search") },
                        selected = currentRoute == LibraryRoutes.Add.route,
                        onClick = { navController.navigate(LibraryRoutes.Add.route) }
                    )
                }
            }
        }
    ) { innerPadding ->
        NavHost(
            navController = navController,
            startDestination = LibraryRoutes.List.route,
            modifier = Modifier.padding(innerPadding),
            contentAlignment = Alignment.Center
        ) {
            composable(LibraryRoutes.List.route){
                GameList(onCardClick = {
                    navController.navigate("game/$it")
                })
            }
            composable(LibraryRoutes.Add.route) {
                SearchList(onCardClick = {
                    navController.navigate("game/$it")
                })
            }
            composable(
                route = LibraryRoutes.Detail.route,
                arguments = listOf(navArgument("gameId") {
                    type = NavType.IntType
                })
            ) {
                backStackEntry ->
                DetailView(
                    navBackStackEntry = backStackEntry,
                    modifier = Modifier
                )
            }
        }
    }
}


@Composable
fun DetailCard(
    modifier: Modifier = Modifier,
    title: String,
    details: String,
    note: String,
    imageUrl: String) {
    Surface(
        modifier = modifier
            .fillMaxWidth()
            .padding(horizontal = 16.dp, vertical = 4.dp)
            .clip(RoundedCornerShape(12.dp))
            .border(1.dp, Color.LightGray, RoundedCornerShape(12.dp))
    ) {
        Column(modifier) {
            ShowPicture(modifier = Modifier.height(200.dp), imageUrl)
            LayoutMediaText(modifier = Modifier, title, details)
        }
    }
}

@Composable
fun HorizontalCard(
    modifier: Modifier = Modifier,
    title: String,
    details: String,
    imageUrl: String,
    onClick: () -> Unit) {
    Surface(
        modifier = modifier
            .fillMaxWidth()
            .padding(horizontal = 16.dp, vertical = 4.dp)
            .requiredHeight(80.dp)
            .clip(RoundedCornerShape(12.dp))
            .border(1.dp, Color.LightGray, RoundedCornerShape(12.dp))
            .clickable { onClick() }
    ) {
        Row(modifier = Modifier.fillMaxSize()) {
            LayoutMediaTextHor(modifier = Modifier.weight(1f), title, details)
            ShowPicture(modifier = Modifier.weight(0.5f), imageUrl)
        }
    }
}

@Composable
fun LayoutMediaText(modifier: Modifier = Modifier, title: String, details: String) {
    Row(
        verticalAlignment = Alignment.CenterVertically,
        modifier = modifier
            .fillMaxWidth()
            .clip(shape = RoundedCornerShape(12.dp))
    ) {
        Row(
            horizontalArrangement = Arrangement.spacedBy(16.dp, Alignment.Start),
            verticalAlignment = Alignment.CenterVertically,
            modifier = Modifier
                .padding(all = 16.dp)
        ) {
            Column(
                verticalArrangement = Arrangement.spacedBy(4.dp, Alignment.Top),
                modifier = Modifier
            ) {
                Text(
                    text = title,
                    color = Color.DarkGray,
                    lineHeight = 1.5.em,
                    style = androidx.compose.ui.text.TextStyle(
                        fontSize = 16.sp,
                        fontWeight = FontWeight.Medium,
                        letterSpacing = 0.15.sp
                    ),
                    modifier = Modifier
                        .fillMaxWidth())
                Text(
                    text = details,
                    color = Color.DarkGray,
                    lineHeight = 1.43.em,
                    style = androidx.compose.ui.text.TextStyle(
                        fontSize = 14.sp,
                        letterSpacing = 0.25.sp
                    ))
            }
        }
    }
}

@Composable
fun LayoutMediaTextHor(modifier: Modifier = Modifier, title: String, details: String) {
    Row(
        verticalAlignment = Alignment.CenterVertically,
        modifier = modifier
            .fillMaxWidth()
            .clip(shape = RoundedCornerShape(12.dp))
    ) {
        Row(
            horizontalArrangement = Arrangement.spacedBy(16.dp, Alignment.Start),
            verticalAlignment = Alignment.CenterVertically,
            modifier = Modifier
                .fillMaxSize()
                .weight(weight = 1f)
                .padding(all = 16.dp)
        ) {
            Column(
                verticalArrangement = Arrangement.spacedBy(4.dp, Alignment.Top),
                modifier = Modifier
                    .fillMaxWidth()
            ) {
                Text(
                    text = title,
                    color = Color.DarkGray,
                    lineHeight = 1.5.em,
                    style = androidx.compose.ui.text.TextStyle(
                        fontSize = 16.sp,
                        fontWeight = FontWeight.Medium,
                        letterSpacing = 0.15.sp
                    ),
                    modifier = Modifier
                        .fillMaxWidth())
                Text(
                    text = details,
                    color = Color.DarkGray,
                    lineHeight = 1.43.em,
                    style = androidx.compose.ui.text.TextStyle(
                        fontSize = 14.sp,
                        letterSpacing = 0.25.sp
                    ),
                    modifier = Modifier
                        .fillMaxWidth())
            }
        }
    }
}

@Composable
fun ShowPicture(modifier: Modifier = Modifier, imageUrl: String) {
    Surface(
        shape = RoundedCornerShape(12.dp),
        color = Color(0xfffef7ff),
        modifier = modifier
            .clip(shape = RoundedCornerShape(12.dp))
    ) {
        //Display Image here
        AsyncImage(
            model = imageUrl,
            contentDescription = "Translated description of what the image contains",
            Modifier.fillMaxHeight(),
            contentScale = ContentScale.Crop
        )
    }
}

@Preview(widthDp = 360, heightDp = 80)
@Composable
private fun HorizontalCardPreview() {
    HorizontalCard(Modifier, "Uncharted 3", "Action Adventure, 2011", "https://media.rawg.io/media/games/b7f/b7ffc4c4776e61eca19d36d3c227f89a.jpg", onClick = {
        println("clicked")
    })
}

@Preview
@Composable
private fun DetailCardPreview() {
    DetailCard(Modifier, "Uncharted 3", "Action Adventure, 2011", "test", "https://media.rawg.io/media/games/b7f/b7ffc4c4776e61eca19d36d3c227f89a.jpg")
}